package com.logicbig.tests.json;

import javax.json.*;
import java.io.StringReader;


public class JsonTypeExamples {
    public static void main(String... strings) {
        // arrayExample();
        //  objectExample();

        numberExample();
        // stringExample();
        //  booleanExmaple();
        //  nullExample();

    }

    private static void nullExample() {
        JsonReader reader = Json.createReader(
                new StringReader("{\"name\": \"Mike\", \"address\": null}"));
        JsonStructure theStructure = reader.read();
        if (theStructure instanceof JsonObject) {
            JsonValue jsonValue = ((JsonObject) theStructure).get("address");
            if (jsonValue == JsonValue.NULL) {
                System.out.println("address is " + jsonValue);
            }
        }
    }

    private static void booleanExmaple() {
        JsonReader reader = Json.createReader(
                new StringReader("{\"name\": \"Mike\", \"married\": false}"));
        JsonStructure theStructure = reader.read();
        if (theStructure instanceof JsonObject) {
            JsonValue jsonValue = ((JsonObject) theStructure).get("married");
            /*you will notice a difference here, but you can still do the something
             like jsonValue.getValueType() == JsonValue.ValueType.TRUE,
             I think getValueType is little inconsistent approach, they should
             have created a new class JSONBoolean instead and we could just use
             instanceof operator all the time.
             */
            if (jsonValue == JsonValue.TRUE || jsonValue == JsonValue.FALSE) {
                System.out.println("married = " + jsonValue);
            }
        }
    }

    private static void stringExample() {
        JsonReader reader = Json.createReader(
                new StringReader("{\"message\": \"hi there!\"}"));
        /*we are going to be type safe at this point as compare
        to the last example*/
        JsonStructure theStructure = reader.read();
        if (theStructure.getValueType() == JsonValue.ValueType.OBJECT) {
            JsonValue jsonValue = ((JsonObject) theStructure).get("message");
            if (jsonValue.getValueType() == JsonValue.ValueType.STRING) {
                System.out.println("message = " +
                        ((JsonString) jsonValue).getString());
            }
        }
    }

    private static void numberExample() {
        JsonReader reader = Json.createReader(
                /*just for example, in real application we will
                be reading it from external source*/
                new StringReader("{\"name\": \"Mike\", \"age\": 23, " +
                        "\"height\":1.8034}"));

        JsonObject jsonObject = reader.readObject();

        //not type safe in real scenarios
        int age = jsonObject.getInt("age");
        System.out.println("age = " + age);

        //to be type safe this time
        JsonValue jsonValue = jsonObject.get("height");
        if (jsonValue instanceof JsonNumber) {/*alternatively
        jsonValue.getValueType()== JsonValue.ValueType.NUMBER*/
            System.out.println("height = " + ((JsonNumber) jsonValue)
                    .bigDecimalValue());
        }
    }

    private static void objectExample() {
        JsonObjectBuilder theBuilder = Json.createObjectBuilder();
        theBuilder.add("name", "Mike");
        theBuilder.add("age", 23);
        JsonObject jsonObject = theBuilder.build();
        Json.createWriter(System.out).write(jsonObject);
    }

    private static void arrayExample() {
        JsonArrayBuilder theBuilder = Json.createArrayBuilder();

        theBuilder.add(
                Json.createObjectBuilder()
                        .add("name", "Mike")
                        .add("age", 23));
        theBuilder.add(
                Json.createObjectBuilder()
                        .add("name", "John")
                        .add("age", 30));
        JsonArray jsonArray = theBuilder.build();

        Json.createWriter(System.out).write(jsonArray);

    }
}
